
OmniTool = class()

function OmniTool:init(scene, volumeToAffect, grids, raycastCamera)
    self.raycastCamera = raycastCamera
    self.grids = grids
    self.scene = scene
    self.volume = volumeToAffect
    local sizeX, sizeY, sizeZ = self.volume:size()
    self.volSize = vec3(sizeX, sizeY, sizeZ)
end

function OmniTool:touched(touch)       
    local volumePosition, face = self:raycast(touch.x, touch.y, false)
    if volumePosition ~= nil then
        print("coordinates touched: "..vec3ToString(volumePosition))
    else
        print("raycast returned nothing")
    end 
    return false
end

function OmniTool:raycast(x,y,z)
    -- Initialize variables to hold raycast results
    local blockCoord = nil
    local blockFace = nil
    local gridTouched = false
    
    -- callback for everything touched by raycast until "true" returned
    local grabDetailsIfTouched = function(coord, idInCallback, face)
        
        -- Check if the coord is within the volume bounds
        local coordInVolume = (coord.x >= -1 and coord.x <= self.volSize.x and 
        coord.y >= -1 and coord.y <= self.volSize.y and
        coord.z >= -1 and coord.z <= self.volSize.z)
        
        --if so, see if a grid was touched
        if coordInVolume then
            for k, v in pairs(self.grids) do
                if v.enabled and v:isVisible() then
                    local vector = coord + face - v.origin
                    local d = math.abs(v.normal:dot(vector))
                    gridTouched = (d == 0)
                    -- set block information if a grid is touched
                    if gridTouched then
                        print("gridTouched: "..k.."\nproduct: ", d, "\nnormal: "..vec3ToString(v.normal).."\nvector: "..vec3ToString(vector))
                        blockCoord = coord
                        blockFace = face  
                        return true  -- Stop raycasting once a grid is touched
                    end
                end
            end
        end
        return false  -- Continue raycasting if neither grid nor block touched yet
    end
    
    -- Convert screen coordinates to a ray origin and direction
    local origin, dir = self.raycastCamera:screenToRay(vec2(x, y))
    -- Perform raycasting using origin, dir, and the defined callback
    self.volume:raycast(origin, dir, 128, grabDetailsIfTouched)
    
    --output results
    print("screen x, y: "..x..", "..y.."\norigin: "..vec3ToString(origin).."\ndir: "..vec3ToString(dir).."\nblockCoord: "..vec3ToString(blockCoord).."\nblockFace: "..vec3ToString(blockFace))  
    
    return blockCoord, blockFace
end

function vec3ToString(v)
    if v == nil then return "nil" end
    local x = string.format("%.1f", v.x)
    local y = string.format("%.1f", v.y)
    local z = string.format("%.1f", v.z)
    return string.format("(%s, %s, %s)", x, y, z)
end